==> MoinLight-0.1.tar.bz2: MoinLight-0.1/moinformat/macros/toc.py <==
#!/usr/bin/env python

"""
Table of contents macro.

Copyright (C) 2018, 2019 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.macros.common import Macro
from moinformat.tree.moin import Block, Container, Heading, Link, LinkLabel, \
                                 List, ListItem, Text
from moinformat.utils.links import parse_link_target

def in_range(min_level, level, max_level):

    """
    Test that 'min_level' <= 'level' <= 'max_level', only imposing tests
    involving limits not set to None.
    """

    return (min_level is None or min_level <= level) and \
           (max_level is None or level <= max_level)

def above_minimum(min_level, level, max_level):

    """
    Test that 'min_level' < 'level' <= 'max_level', only imposing tests
    involving limits not set to None.
    """

    return (min_level is None or min_level < level) and \
           (max_level is None or level <= max_level)

class TableOfContents(Macro):

    "A table of contents macro."

    name = "TableOfContents"

    def evaluate(self):

        "Evaluate the macro, producing a table of contents."

        arglist = []
        _defaults = [None] * 2

        for arg, default in map(None, self.node.args, _defaults):
            if arg is not None:
                try:
                    arg = max(1, int(arg.strip()))
                except ValueError:
                    arg = None
            arglist.append(arg)

        self.make_table(arglist[0], arglist[1])

    def make_table(self, min_level=None, max_level=None):

        """
        Make a table of contents with the given 'min_level' and 'max_level' of
        headings.
        """

        headings = []
        self.find_headings(self.doc, headings)

        if not headings:
            return

        # Common list features.

        marker = "1."
        space = " "
        num = "1"

        # Start with no lists, no current item.

        lists = []
        item = None
        level = 0

        for heading in headings:
            new_level = heading.level

            # Create new lists if the level increases.

            if new_level > level:
                while level < new_level:
                    level += 1

                    # Ignore levels outside the range of interest.

                    if not in_range(min_level, level, max_level):
                        continue

                    # Determine whether the heading should be generated at this
                    # level or whether there are intermediate levels being
                    # produced.

                    nodes = level == new_level and self.get_entry(heading) or []
                    indent = level - 1

                    # Create a new item for the heading or sublists.

                    new_item = ListItem(nodes, indent, marker, space, None)

                    # Either revive an existing list.

                    if level == min_level and lists:
                        new_list = lists[-1]
                        new_items = new_list.nodes

                    # Or make a list and add an item to it.

                    else:
                        new_items = []
                        new_list = List(new_items)

                        # Add the list to the current item, if any.

                        if item:
                            item.nodes.append(new_list)

                        # Record the new list.

                        lists.append(new_list)

                    # Add the item to the new or revived list.

                    new_items.append(new_item)

                    # Reference the new list's items and current item.

                    items = new_items
                    item = new_item

            else:
                # Retrieve an existing list if the level decreases.

                if new_level < level:
                    while level > new_level:

                        # Retain a list at the minimum level.

                        if above_minimum(min_level, level, max_level):
                            lists.pop()

                        level -= 1

                    # Obtain the existing list and the current item.

                    items = lists[-1].nodes
                    item = items[-1]

                # Add the heading as an item.

                if in_range(min_level, level, max_level):

                    indent = level - 1
                    nodes = self.get_entry(heading)

                    item = ListItem(nodes, indent, marker, space, None)
                    items.append(item)

        # Augment the macro node with the top-level list.

        self.insert_table(lists[0])

    def insert_table(self, content):

        "Insert the given 'content' into the document."

        macro = self.node
        parent = macro.parent
        region = macro.region

        # Set the children of this macro with the generated content.

        macro.nodes = [content]
        macro.inline = False

        # Split any block containing the macro into preceding and following
        # parts. 

        if isinstance(parent, Block):
            following = parent.split_at(macro)

            # Insert any non-empty following block.

            if not following.whitespace_only():
                region.insert_after(parent, following)

            # Insert the new content.

            region.insert_after(parent, macro)

            # Remove any empty preceding block.

            if parent.whitespace_only():
                region.remove(parent)

    def find_headings(self, node, headings):

        "Find headings under 'node', adding them to the 'headings' list."

        if node.nodes:
            for n in node.nodes:
                if isinstance(n, Heading):
                    headings.append(n)
                elif isinstance(n, Container):
                    self.find_headings(n, headings)

    def get_entry(self, heading):

        "Return nodes for an entry involving 'heading'."

        target = "#%s" % heading.identifier
        link_target = parse_link_target(target, self.metadata)
        link_label = LinkLabel(heading.nodes[:])

        return [Link([link_label], link_target), Text("\n")]

macro = TableOfContents

# vim: tabstop=4 expandtab shiftwidth=4

==> MoinLight-0.2.1.tar.bz2: MoinLight-0.2.1/moinformat/macros/toc.py <==
#!/usr/bin/env python

"""
Table of contents macro.

Copyright (C) 2018, 2019, 2022 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.macros.common import Macro
from moinformat.tree.moin import Block, Container, Heading, Link, LinkLabel, \
                                 List, ListItem, Text
from moinformat.utils.links import parse_link_target

def in_range(min_level, level, max_level):

    """
    Test that 'min_level' <= 'level' <= 'max_level', only imposing tests
    involving limits not set to None.
    """

    return (min_level is None or min_level <= level) and \
           (max_level is None or level <= max_level)

def above_minimum(min_level, level, max_level):

    """
    Test that 'min_level' < 'level' <= 'max_level', only imposing tests
    involving limits not set to None.
    """

    return (min_level is None or min_level < level) and \
           (max_level is None or level <= max_level)

class TableOfContents(Macro):

    "A table of contents macro."

    name = "TableOfContents"

    def evaluate(self):

        "Evaluate the macro, producing a table of contents."

        arglist = []
        _defaults = [None] * 2

        for arg, default in map(None, self.node.args, _defaults):
            if arg is not None:
                try:
                    arg = max(1, int(arg.strip()))
                except ValueError:
                    arg = None
            arglist.append(arg)

        self.make_table(arglist[0], arglist[1])

    def make_table(self, min_level=None, max_level=None):

        """
        Make a table of contents with the given 'min_level' and 'max_level' of
        headings.
        """

        headings = []
        self.find_headings(self.doc, headings)

        if not headings:
            return

        # Common list features.

        marker = "1."
        space = " "
        num = "1"

        # Start with no lists, no current item.

        lists = []
        item = None
        level = 0

        for heading in headings:
            new_level = heading.level

            # Create new lists if the level increases.

            if new_level > level:
                while level < new_level:
                    level += 1

                    # Ignore levels outside the range of interest.

                    if not in_range(min_level, level, max_level):
                        continue

                    # Determine whether the heading should be generated at this
                    # level or whether there are intermediate levels being
                    # produced.

                    nodes = level == new_level and self.get_entry(heading) or []
                    indent = level - 1

                    # Create a new item for the heading or sublists.

                    new_item = ListItem(nodes, indent, marker, space, None)

                    # Either revive an existing list.

                    if level == min_level and lists:
                        new_list = lists[-1]
                        new_items = new_list.nodes

                    # Or make a list and add an item to it.

                    else:
                        new_items = []
                        new_list = List(new_items)

                        # Add the list to the current item, if any.

                        if item:
                            item.nodes.append(new_list)

                        # Record the new list.

                        lists.append(new_list)

                    # Add the item to the new or revived list.

                    new_items.append(new_item)

                    # Reference the new list's items and current item.

                    items = new_items
                    item = new_item

            else:
                # Retrieve an existing list if the level decreases.

                if new_level < level:
                    while level > new_level:

                        # Retain a list at the minimum level.

                        if above_minimum(min_level, level, max_level):
                            lists.pop()

                        level -= 1

                    # Obtain the existing list and the current item.

                    items = lists[-1].nodes
                    item = items[-1]

                # Add the heading as an item.

                if in_range(min_level, level, max_level):

                    indent = level - 1
                    nodes = self.get_entry(heading)

                    item = ListItem(nodes, indent, marker, space, None)
                    items.append(item)

        # Augment the macro node with the top-level list.

        self.insert_table(lists and lists[0] or None)

    def insert_table(self, content):

        "Insert the given 'content' into the document."

        macro = self.node
        parent = macro.parent
        region = macro.region

        # Set the children of this macro with the generated content.

        macro.nodes = content and [content]
        macro.inline = False

        # Split any block containing the macro into preceding and following
        # parts. 

        if isinstance(parent, Block):
            following = parent.split_at(macro)

            # Insert any non-empty following block.

            if not following.whitespace_only():
                region.insert_after(parent, following)

            # Insert the new content.

            region.insert_after(parent, macro)

            # Remove any empty preceding block.

            if parent.whitespace_only():
                region.remove(parent)

    def find_headings(self, node, headings):

        "Find headings under 'node', adding them to the 'headings' list."

        if node.nodes:
            for n in node.nodes:
                if isinstance(n, Heading):
                    headings.append(n)
                elif isinstance(n, Container):
                    self.find_headings(n, headings)

    def get_entry(self, heading):

        "Return nodes for an entry involving 'heading'."

        target = "#%s" % heading.identifier
        link_target = parse_link_target(target, self.metadata)
        link_label = LinkLabel(heading.nodes[:])

        return [Link([link_label], link_target), Text("\n")]

macro = TableOfContents

# vim: tabstop=4 expandtab shiftwidth=4
